// Persistence of Vision Ray Tracer Scene Include File
// File: sunpos.inc
// Desc: This macro returns the suns position,          
//       on a given date, time, and location on earth.  
// Date: 15-09-1998
// Updated : 2001-07-27
// Auth: Ingo Janssen
//
// Updated 2004-03-14 by Stephen Parke
// Implemented rev(*) function inline instead of if-statements
// Corrected example to align and object with the sunlight
// Added debug statements and test data for sample date 1990-04-19 from original text
//
// ====================================================== 
//
// Assumption: in the scene north is in the +Z direction, south is -Z.
//
// Invoke in your scene:
//   light_source {
//      SunPos(Year, Month, Day, Hour, Minute, Lstm, LAT, LONG)
//      rgb 1
//   }
//
// Greenwich, noon on the longest day of 2000 (no daylight saving):
//   SunPos(2000, 6, 21, 12, 2, 0, 51.4667, 0.00) 
//
// Year:  in four digits
// Month: number
// Dat:   number
// Time:  in 24 hour format
// Lstm:  Meridian of your local timezone (+1 hour =  +15 deg)
//        in degrees (east = positive, west = negative)
// Location on earth: 
// LAT:   Lattitude in degrees.decimal, northern hemisphere = positive, southern = negative
// LONG:  Longitude in degrees.decimal, east = positive, west is negative
//
// The macro returns the position of the sun, but also declares it as the vector SolarPosition. 
// Also the found Azimuth (Az) and Altitude (Al) are declared, this can be usefull for 
// aligning an object with the sunlight: cylinder{
//                                          <-2,0,0>,<2,0,0>,0.1
//                                          rotate <0, Az-90, Al>   
//                                          rotate <0, 176-90, Al>  THIS WORKS AND ROTATES UP LIKE THE SUN DOES
//                                          rotate <0, 186-90, Al>  THIS FAILS BECAUSE IT DROPS BELOW THE XZ PLANE
//                                          texture {.. LightRay ..}
//                                        }
// 
// Try this instead:      ... aligning an object with the sunlight: 
//                        #declare Length = 1.0;
//                        #declare SunBeamVector = vrotate(<0, 0, Length>,<-Al, Az, 0>); 
//                        cylinder{ < 0,0,0 >, SunBeamVector, 0.1  pigment {Yellow}  } 
//
// Note - this cylinder will be oriented with the origin and the sun.
// Caution should be used to ensure the cylinder does not block the camera,
// particularly when the camera is looking at the origin but is closer than Length.
//
//
// ======================================================
//
// Find your local position at http://gnpswww.nima.mil/geonames/GNS/ (Gazetteer search)
//
// Local time(zone) at http://www.hilink.com.au/times/
//
// Equations used here can be found at http://hotel04.ausys.se/pausch/english.htm (Computing rise/set times)
//  not only for the sun but also for the moon, planets and other stuff up there.
//
// ======================================================
//

#macro SunPos2(Year, Month, Day, Hour, Minute, Lstm, LAT, LONG)

   #debug concat("\nStart SunPos Macro: ",str(Year,4,0),"/", str(Month,-2,0),"/",str(Day,-2,0),"  at ", str(Hour,-2,0),":", str(Minute,-5,2),"\n")   
   
   #if (abs(LONG-Lstm)>30)
   //#debug "\nREMARK: \nLongitude differs by more than 30 degrees from timezone meridian.\n"
   //#debug concat("Local timezone meridian is:",str(Lstm,5,0),"\n")
   //#debug concat("Longitude is:",str(LONG,5,0),"\n")
   #end
   
// Calculate universal time (UT)
   #local T= Hour+(Minute/60);
   #local UT= T-Lstm/15;
   #if (0>UT)
       #local Day= Day-1;
       #local UT= 24+UT;
   #end
   #if (UT>24)
       #local Day= Day+1;
       #local UT= UT-24;
   #end          

   //#debug concat("      SunPos Macro: ",str(Year,4,0),"/", str(Month,-2,0),"/",str(Day,-2,0),"  at ", str(Hour,-2,0),":", str(Minute,-5,2),"\n\n")   
   //#debug concat("(0)Local UT      is: ",str(UT,10,4),"\n")   

// Sample date from text is April 19, 1990  (1990, 04, 19)              // sample d = -3543.0
// Amount of days to, or from, the year 2000  
   #local d= 367*Year-int((7*(Year+int((Month+9)/12)))/4)+int((275*Month)/9)+Day-730530+UT/24.0;  
   //#debug concat("(0)Local d       is: ",str(d,10,4),"\n")
 
// Longitude of perihelion (w), eccentricity (e)
   #local w= 282.9404+4.70935E-5*d;                                     // sample w = 282.7735_deg
   #local e= 0.016709-1.151E-9*d;                                       // sample e = 0.016713
   
// Mean anomaly (M)  rev(M)
   #local M= 356.0470+0.9856002585*d;   
   //#debug concat("(1)Local M       is: ",str(M,10,4),"\n")              // sample M = -3135.9347_deg
   #local M= M-floor(M/360)*360;    
   //#debug concat("(2)Local M       is: ",str(M,10,4),"\n")              // rev(M) = 104.0653_deg
 
// Sun's mean lontitude (L)  rev(L)
   #local L= w+M;
   //#debug concat("(0)Local L       is: ",str(L,10,4),"\n")              // sample L = 386.8388_deg
   #local L= L-floor(L/360)*360; 
   //#debug concat("(0)Local L       is: ",str(L,10,4),"\n")              // rev(L) = 26.83.88_deg
          
// Obliquity of the ecliptic, 
   #local oblecl= 23.4393-3.563E-7*d;
   //#debug concat("(0)Local oblecl  is: ",str(oblecl,10,4),"\n")         // sample oblecl is 23.4406_deg   
     
// Eccentric anomaly (E) 
   #local E= M+(180/pi)*e*sin(radians(M))*(1.0+e*cos(radians(M)));      // sample E is 104.9904_deg
 
// Sun's rectangular coordinates in the plane of ecliptic (A,B)
// these are usually referred to as X&Y but don't try to confuse povray vectors
   #local A= cos(radians(E))-e;                                         // sample A = ~ -0.28
   #local B= sin(radians(E))*sqrt(1-e*e);                               // sample B = ~  0.96                                          
                                          
// Distance (r), true anomaly (V)
   #local r= sqrt(A*A+B*B);                                             // sample r = ~ 1.00
   #local V= degrees(atan2(radians(B),radians(A)));                     // sample v = ~ 105.9  

// Longitude of the sun (slon)
   #local slon= V+w;
   //#debug concat("(1)Local slon    is: ",str(slon,10,4),"\n")           // sample slon = 288.6869_deg
   #local slon= slon-floor(slon/360)*360; 
   //#debug concat("(2)Local slon    is: ",str(slon,10,4),"\n")           // rev(slon) = 28.68.6869_deg

// Calculate declination (Decl) 
   #local Decl= degrees(asin(sin(radians(oblecl))*sin(radians(slon))));  
   //#debug concat("(0)Local Decl    is: ",str(Decl,10,4),"\n")           // value is in degrees
   
// Calculate right ascension (RA) and convert to hours by /15
   #local RA= degrees ( atan2 ( sin(radians(slon)) * cos(radians(oblecl)), cos(radians(slon)) ) )/15;
   //#debug concat("(0)Local RA      is: ",str(RA,10,4),"\n")             // value is in hours 
      
// Greenwich meridian sidereal time at 00:00 (GMST0),sidereal time (SIDTIME))
   #local GMST0= L/15+12;                                              
   //#debug concat("(0)Local GMST0   is: ",str(GMST0,10,4),"\n")          // value is in hours 

// Make sure time originally specified is LOCAL time and LSTM is set appropriately
   #local SIDTIME= GMST0+UT+LONG/15; 
   //#debug concat("(0)Local SIDTIME is: ",str(SIDTIME,10,4),"\n")        // value is in hours 
   
// Local hour angle, convert to degrees, rev(LHA)
   #local LHA = (SIDTIME-RA)*15;
   //#debug concat("(1)Local LHA     is: ",str(LHA,10,4),"\n")            // value is in degrees
   #local LHA= LHA-floor(LHA/360)*360; 
   //#debug concat("(2)Local LHA     is: ",str(LHA,10,4),"\n\n")          // value is in degrees 


// This is what we're looking for: Altitude & Azimuth
   #declare Al= degrees( asin ( sin(radians(LAT)) * sin(radians(Decl)) + cos(radians(LAT)) * cos(radians(Decl)) * cos(radians(LHA)) ) );
   #declare Az= degrees	( atan2	( sin(radians(LHA))*cos(radians(Decl)), cos(radians(LHA)) * cos(radians(Decl)) * sin(radians(LAT)) - sin(radians(Decl)) * cos(radians(LAT)) ) ) + 180;
   #declare SolarPosition=vrotate(<0,0,1000000000>,<-Al,Az,0>);
   (SolarPosition)

   //#debug concat("Al_deg                   is: ",str(Al,10,4),"\n") 
   //#debug concat("Az_deg                   is: ",str(Az,10,4),"\n\n") 


#end
