//**********************************************************************
// Persistence of Vision Ray Tracer Scene Include File
// File: sunriset.inc 
// Vers: 3.5
// Desc: This macro returns the suns position,          
//       for sunrise, sun in due south and sunset.
// Date: 2004-16-02
// Auth: Stephen Parke based on the POVRay work SunPos() by Ingo Janssen
//       and the mathematical work of the original author Paul Schlyter.  
//       whose website has moved to 
//       http://www.stjarnhimlen.se/english.html 
//       where he provides the 'C' source for rise/set calculations.
//
//
//
// For Calgary, Canada, on our shortest day of the year,
// almost the same latitude as Greenwich, although timezone is GMT -7
// Ensure the time specified is LOCAL and Lstm is specified in _degrees_  
// Daylight savings time is NOT considered so reference standard time.
// 
//  #declare Year=    2003; // Year:  in four digits
//  #declare Month=     12; // Month: number
//  #declare Day=       21; // Day:   number
//  #declare Hour=      00; // UNUSED BY SunRiSet (Noon becomes assumed for calculation purposes)
//  #declare Minute=    00; // UNUSED BY SunRiSet
//  #declare Lstm=   -7*15; // Local time zone in degrees (east = positive, west = negative)
//  #declare LAT=       51; // Latitude in degrees.decimal, northern hemisphere = positive, southern = negative
//  #declare LONG=    -114; // Longtitude in degrees.decimal, east = positive, west is negative    
//  #declare Angle= -35/60; // Number of degrees above the horizon for sunrise and sunset:
//   
// Invoke the macro to set the variables and then call SunPos() with the 
// resulting calculated Hours and Minutes for the solar position of interest.
// NOTE : this macro does not HAVE to be used with sunpos(), 
// but was built to limit a scene to the minutes between sunrise and sunset
// when illuminating a scene with the sun defined by the sunpos() position.
// 

//
/* SunRiSet(Year, Month, Day, Lstm, LAT, LONG, Angle) */
//
// which sets global variables in local time
//   SunRise
//   SunSouth
//   SunSet 
//

//
// Uncomment one of the following paragraphs for stills, or for animations
//

////// SAMPLE CODE FOR STILL PICTURES //////
// For still picture with sun exactly South:
// The macro calculates the hour and minute
// of SunSouth (where the sun is exactly south  
// ( -z ) or 180_compass_deg at appx Noon local time
// which varies depending upon how far across the 
// timezone the local position is.
//
//#declare Hour = int(SunSouth);
//#declare Minute = (SunSouth - Hour) * 60;
//
// Redeclare Hour and minute based on returned value for Sun in South (noonish)
//#declare Hour = int(SunSouth);                          // set hour to hour of sun in south
//#declare Minute = int((SunSouth - Hour) * 60);          // set minute to minute when sun is in south
// If desired, any other time can happily be used, such as:
//#declare Hour = int(SunRise);                           // set hour to hour of sunrise
//#declare Minute = (SunRise - Hour) * 60;                // set minute to minutes of sunrise 
// or
//#declare Hour = int(SunSet);
//#declare Minute = int((SunSet - Hour) * 60);
//
////#debug concat("Source file Hour:Minute is      ",str(Hour,-2,0),":",str(Minute,-2,0), "\n\n")
//

//
////// SAMPLE CODE FOR ANIMATIONS DURING DAYLIGHT HOURS ONLY //////
// For animation:     e.g.    +kff60
// Build an animation of any number of frames 
// This math calculates the number of minutes 
// between (SunSet - SunRise) which is used   
// to adjust the clock accordingly.   
//
// ANIMATION DURING DAYLIGHT HOURS ONLY //
// Redeclare Hour and minute based on returned value for SunRise and SunSet
//#declare Hour = int(SunRise);                           // set starting hour to hour of sunrise
//#declare Minute = (SunRise - Hour) * 60;                // set starting minute to minutes of sunrise 
//
//#declare ClockMinutes = (SunSet - SunRise) * 60;        // total number of minutes of daylight between sunrise and sunset 
//#declare Minute=   Minute + int(clock * ClockMinutes);  // increment minutes based on animation clock value
//#declare Hour = Hour + int(Minute/60);                  // update hour to new animation value   
//#declare Minute = Minute-int(Minute/60)*60;             // update minute to balance of animation value  0..59
//
////#debug concat("\nSource file ClockMinutes is :",str(ClockMinutes,8,5),"\n") 
////#debug concat("Source file Hour is         :",str(Hour,-2,0),"\n")
////#debug concat("Source file Minute is       :",str(Minute,-2,0),"\n\n")
//  

//
// Refer to SunPos() for additional setup information
//
// ====================================================== 
//
// Assumption: in the scene north is in the +Z direction, south is -Z.
// Therefore, west is -x, east is +x, and +y, is, well, UP (antigravity).
//
// Place the light source in your scene
//   light_source {
//      SunPos(Year, Month, Day, Hour, Minute, Lstm, LAT, LONG)
//      rgb 1
//   }
//
// ======================================================
//
 
#macro SunRiSet(Year, Month, Day, Lstm, LAT, LONG, Angle)

   //#debug concat("\nStart SunRiSet Macro: ",str(Year,4,0),"/", str(Month,-2,0),"/",str(Day,-2,0), "\n\n")  
   
   #if (abs(LONG-Lstm)>30)
   //#debug "\nREMARK: \nLongitude differs by more than 30 degrees from timezone meridian.\n"
   //#debug concat("Local timezone meridian is:",str(Lstm,5,0),"\n")
   //#debug concat("Longitude is:",str(LONG,10,4),"\n")
   #end
   
   // Calculate universal time (UT) at the local timezone
   #local UT= Lstm/15;
   //#debug concat("(0)Local UT          is: ",str(UT,10,4),"\n")  
   
   // Amount of days to, or from, the year 2000
   // Set d to number of days plus one-half, i.e. Noon
   #local d= 367*Year-int((7*(Year+int((Month+9)/12)))/4)+int((275*Month)/9)+Day-730530+ 12.0/24.0;    
   //#debug concat("(0)Local d           is: ",str(d,10,4),"\n")

   // Longitude of perihelion (w), eccentricity (e)
   #local w= 282.9404+4.70935E-5*d;
   #local e= 0.016709-1.151E-9*d;
   
   // Mean anomaly (M)  rev(M)
   #local M= 356.0470+0.9856002585*d;   
   //#debug concat("(1)Local M           is: ",str(M,10,4),"\n") 
   #local M= M-floor(M/360)*360;   
   //#debug concat("(2)Local M           is: ",str(M,10,4),"\n")
     
   // Sun's mean longitude (L)  rev(L)
   #local L= w+M;
   //#debug concat("(1)Local L           is: ",str(L,10,4),"\n")
   #local L= L-floor(L/360)*360; 
   //#debug concat("(2)Local L           is: ",str(L,10,4),"\n")
           
   // Obliquity of the ecliptic
   #local oblecl= 23.4393-3.563E-7*d;
   //#debug concat("(0)Local oblecl      is: ",str(oblecl,10,4),"\n")   
   
   // eccentric anomaly (E)
   #local E= M+e*(180/pi)*sin(radians(M))*(1.0+e*cos(radians(M)));  
   //#debug concat("(0)Local E           is: ",str(E,10,4),"\n")   
   
   // Sun's rectangular coordinates in the plane of ecliptic (A,B)
   #local A= cos(radians(E))-e;  
   #local B= sin(radians(E))*sqrt(1-e*e);
                                          
   // Distance (r), true anomaly (V)
   #local r= sqrt(A*A+B*B);                   
   #local V= degrees(atan2(radians(B),radians(A)));  

   // Longitude of the sun (slon) rev(slon)
   #local slon= V+w;
   //#debug concat("(1)Local slon        is: ",str(slon,10,4),"\n")
   #local slon= slon-floor(slon/360)*360; 
   //#debug concat("(2)Local slon        is: ",str(slon,10,4),"\n")   
                            
   // Calculate declination (Decl) and right ascension (RA) values are in degrees (not hours)      
   #local Decl_deg = degrees(asin(sin(radians(oblecl))*sin(radians(slon))));
   #local RA_deg = degrees ( atan2 ( sin(radians(slon)) * cos(radians(oblecl)), cos(radians(slon)) ) );
   //#debug concat("(0)Local RA          is: ",str(RA_deg,10,4),"\n")
   
   // Greenwich meridian siderial time at 00:00 (GMST0)  rev(GMST0)  in degrees
   #local GMST0= L + 180;
   //#debug concat("(1)Local GMST0_deg   is: ",str(GMST0,10,4),"\n") 
   #local GMST0= GMST0-floor(GMST0/360)*360; 
   //#debug concat("(2)Local GMST0_deg   is: ",str(GMST0,10,4),"\n") 
   // value is in hours degrees

   // siderial time (SIDTIME)  in degress
   #local SIDTIME_deg = GMST0 + 180 + LONG; // don't divide by 15 yet because we are still in degrees
   //#debug concat("(0)Local SIDTIME_deg is: ",str(SIDTIME_deg,10,4),"\n") 

   
   //* Compute time when Sun is at south - in hours UT */
   //* tsouth = 12.0 - rev180(sidtime - sRA)/15.0; */ 
   //*********************************************/
   //* Reduce angle to within +180..+180 degrees */
   //*********************************************/
   // return( x - 360.0 * floor( x * INV360 + 0.5 ) );


   // local hour angle (LHA_deg)  in degrees		
   #local LHA_deg = (SIDTIME_deg - RA_deg) ;  
   //#debug concat("(1)Local LHA_deg     is: ",str(LHA_deg,10,4),"\n") 
   #local LHA_deg= LHA_deg-floor(LHA_deg/360 + 0.5)*360;                              
   //#debug concat("(2)Local LHA_deg     is: ",str(LHA_deg,10,4),"\n") 

   // Finally divide by 15 to get hours
   #declare SunSouth = 12 - LHA_deg / 15;   
  
   #local cosT = (sin(radians(Angle)) - sin(radians(LAT)) * sin(radians(Decl_deg)) ) / (cos(radians(LAT)) * cos(radians(Decl_deg)) );
   // ( cosT <= -1.0 ) = sun above the specified "horizon" 24 hours.     
   //                    SunRise is set to time when the sun is at south time minus 12 hours
   //                    SunSet is set to set to the south time plus 12 hours.
   //                    "Day" length = 24 hours 
   //
   // ( cosT >= +1.0 )   sun is below the specified "horizon" 24 hours 
   //                    "Day" length = 0 hours, SunRise and SunSet are both set to the time when the sun is at south.
   // 
   // ( cosT > -1.0 ) & (cosT < 1.0 ) = sun rises/sets this day, times stored at SunRise and SunSet

   #if ( cosT >= 1.0 )
	#local time = 0.0;                              /* Sun always below altit */
   #end
   #if ( cosT <= -1.0 )
        #local time = 12.0;                             /* Sun always above altit */
   #end
   #if ((cosT > -1.0 ) & (cosT < 1.0))
        #local time = degrees(acos(cosT)) / 15;         /* The diurnal arc, hours */  
   #end 

   //#debug concat("(0)Local cosT        is: ",str(cosT,10,4),"\n") 
   //#debug concat("(0)Local time        is: ",str(time,10,4),"\n\n") 

   #declare SunSouth = SunSouth + UT;    
   #declare SunRise  = SunSouth - time;  
   #declare SunSet   = SunSouth + time;

   #debug concat("SunRise_H_ms  local time is: ",str(SunRise, 10,4),"\n") 
   #debug concat("SunSouth_H_ms local time is: ",str(SunSouth,10,4),"\n") 
   #debug concat("SunSet_H_ms   local time is: ",str(SunSet,  10,4),"\n\n") 

#end



/* This angle computes the length of the day, from sunrise to sunset. */
/* Sunrise/set is considered to occur when the Sun's upper limb is    */
/* 35 arc minutes below the horizon (this accounts for the refraction */
/* of the Earth's atmosphere).                                        */
//#local Angle = -35.0/60.0;

/* This Angle computes the length of the day, including civil twilight. */
/* Civil twilight starts/ends when the Sun's center is 6 degrees below  */
/* the horizon.                                                         */
//#local Angle = -6.0;

/* This angle computes the length of the day, incl. nautical twilight.  */
/* Nautical twilight starts/ends when the Sun's center is 12 degrees    */
/* below the horizon.                                                   */
//#local Angle = -12.0;

/* This angle computes the length of the day, incl. astronomical twilight. */
/* Astronomical twilight starts/ends when the Sun's center is 18 degrees   */
/* below the horizon.                                                      */
//#local Angle = -18.0;
